@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2020 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -9,9 +9,7 @@ import dataclasses
 import enum
 from typing import List, Optional
 
-from simple_parsing import Serializable
-
-import pynguin.utils.statistics.statistics as stat  # pylint:disable=cyclic-import
+from pynguin.utils.statistics.runtimevariable import RuntimeVariable
 
 
 class ExportStrategy(str, enum.Enum):
@@ -36,12 +34,19 @@ class ExportStrategy(str, enum.Enum):
 class Algorithm(str, enum.Enum):
     """Different algorithms supported by Pynguin."""
 
-    RANDOOPY = "RANDOOPY"
+    MOSA = "MOSA"
+    """The many-objective sorting algorithm (cf. Panichella et al. Reformulating Branch
+    Coverage as a Many-Objective Optimization Problem.  Proc. ICST 2015)."""
+
+    RANDOM = "RANDOM"
     """A feedback-direct random test generation approach similar to the algorithm
     proposed by Randoop (cf. Pacheco et al. Feedback-directed random test generation.
     Proc. ICSE 2007)."""
 
-    WSPY = "WSPY"
+    RANDOM_SEARCH = "RANDOM_SEARCH"
+    """Performs random search on test suites."""
+
+    WHOLE_SUITE = "WHOLE_SUITE"
     """A whole-suite test generation approach similar to the one proposed by EvoSuite
     (cf. Fraser and Arcuri. EvoSuite: Automatic Test Suite Generation for
     Object-Oriented Software. Proc. ESEC/FSE 2011)."""
@@ -87,8 +92,8 @@ class StatisticsBackend(str, enum.Enum):
 
 
 # pylint: disable=too-many-instance-attributes, pointless-string-statement
-@dataclasses.dataclass(repr=True, eq=True)
-class Configuration(Serializable):
+@dataclasses.dataclass
+class Configuration:
     """General configuration for the test generator."""
 
     algorithm: Algorithm
@@ -121,10 +126,10 @@ class Configuration(Serializable):
     timeline_interpolation: bool = True
     """Interpolate timeline values"""
 
-    output_variables: List[stat.RuntimeVariable] = dataclasses.field(
+    output_variables: List[RuntimeVariable] = dataclasses.field(
         default_factory=lambda: [
-            stat.RuntimeVariable.TargetModule,
-            stat.RuntimeVariable.Coverage,
+            RuntimeVariable.TargetModule,
+            RuntimeVariable.Coverage,
         ]
     )
     """List of variables to output to the statistics backend."""
@@ -136,8 +141,8 @@ class Configuration(Serializable):
     budget: int = 600
     """Time budget (in seconds) that can be used for generating tests."""
 
-    search_budget: int = 60
-    """Maximum search duration"""
+    maximum_test_number: int = 60
+    """Maximum number of test cases to be created."""
 
     algorithm_iterations: int = 60
     """Maximum iterations"""
@@ -175,6 +180,12 @@ class Configuration(Serializable):
     string_length: int = 20
     """Maximum length of randomly generated strings"""
 
+    bytes_length: int = 20
+    """Maximum length of randomly generated bytes"""
+
+    collection_size: int = 5
+    """Maximum length of randomly generated collections"""
+
     primitive_reuse_probability: float = 0.5
     """Probability to reuse an existing primitive in a test case, if available.
     Expects values in [0,1]"""
@@ -263,21 +274,45 @@ class Configuration(Serializable):
     """Should the generator use a static constant seeding technique to improve constant
     generation?"""
 
-    duck_type_analysis: bool = False
-    """Should the duck-type analysis be used for type inference during test
-    generation?"""
+    initial_population_seeding: bool = False
+    """Should the generator use previously existing testcases to seed the initial
+    population?"""
+
+    initial_population_data: str = ""
+    """The path to the file with the pre-existing tests. The path has to include the
+    file itself."""
+
+    seeded_testcases_reuse_probability: float = 0.9
+    """Probability of using seeded testcases when initial population seeding is
+    enabled."""
+
+    initial_population_mutations: int = 0
+    """Number of how often the testcases collected by initial population seeding should
+    be mutated to promote diversity"""
 
     generate_assertions: bool = True
     """Should assertions be generated?"""
 
+    post_process: bool = True
+    """Should the results be post processed? For example, truncate test cases after
+    statements that raise an exception."""
+
     float_precision: float = 0.01
     """Precision to use in float comparisons and assertions"""
 
-    duck_mock_module_only: bool = False
-    """Do only parse module dependencies for duck mocks, not whole class path."""
+    dynamic_constant_seeding: bool = False
+    """Enables seeding of constants at runtime."""
+
+    seeded_primitives_reuse_probability = 0.9
+    """Probability for using seeded primitive values instead of randomly
+    generated ones."""
+
+    seeded_dynamic_values_reuse_probability = 0.9
+    """Probability of using dynamically seeded values when a primitive seeded
+     value will be used."""
 
 
 # Singleton instance of the configuration.
-INSTANCE = Configuration(
-    algorithm=Algorithm.RANDOOPY, project_path="", output_path="", module_name=""
+configuration = Configuration(
+    algorithm=Algorithm.RANDOM, project_path="", output_path="", module_name=""
 )
