@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2020 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -12,6 +12,7 @@ from abc import abstractmethod
 from statistics import mean
 from typing import Dict, List, Optional
 
+import pynguin.ga.chromosomevisitor as cv
 import pynguin.ga.fitnessfunction as ff
 
 
@@ -28,11 +29,15 @@ class Chromosome(metaclass=abc.ABCMeta):
             self._fitness_values: Dict[ff.FitnessFunction, ff.FitnessValues] = {}
             self._number_of_evaluations: int = 0
             self._changed: bool = True
+            self._distance: float = -1
+            self._rank: int = -1
         else:
             self._fitness_functions = list(orig._fitness_functions)
             self._fitness_values = dict(orig._fitness_values)
             self._number_of_evaluations = orig._number_of_evaluations
             self._changed = orig._changed
+            self._distance = orig._distance
+            self._rank = orig._rank
 
     @abstractmethod
     def size(self) -> int:
@@ -110,6 +115,15 @@ class Chromosome(metaclass=abc.ABCMeta):
             raise RuntimeError(", ".join(violations))
         self._fitness_values[fitness_function] = new_value
 
+    @property
+    def fitness_values(self) -> Dict[ff.FitnessFunction, ff.FitnessValues]:
+        """Provides the registered fitness values.
+
+        Returns:
+            A dictionary from fitness function to fitness value
+        """
+        return self._fitness_values
+
     def add_fitness_function(
         self,
         fitness_function: ff.FitnessFunction,
@@ -200,6 +214,14 @@ class Chromosome(metaclass=abc.ABCMeta):
             The cloned chromosome  # noqa: DAR202
         """
 
+    @abstractmethod
+    def accept(self, visitor: cv.ChromosomeVisitor) -> None:
+        """Accept a chromosome visitor.
+
+        Args:
+            visitor: the visitor that is accepted.
+        """
+
     @abstractmethod
     def __eq__(self, other):
         pass
@@ -207,3 +229,39 @@ class Chromosome(metaclass=abc.ABCMeta):
     @abstractmethod
     def __hash__(self):
         pass
+
+    @property
+    def distance(self) -> float:
+        """Provides the distance value of this chromosome.
+
+        Returns:
+            The distance value of this chromosome
+        """
+        return self._distance
+
+    @distance.setter
+    def distance(self, distance: float) -> None:
+        """Sets the distance value of this chromosome.
+
+        Args:
+            distance: The new distance value
+        """
+        self._distance = distance
+
+    @property
+    def rank(self) -> int:
+        """Provide the rank value of this chromosome.
+
+        Returns:
+            The rank value of this chromosome
+        """
+        return self._rank
+
+    @rank.setter
+    def rank(self, rank: int) -> None:
+        """Sets the rank value of this chromosome.
+
+        Args:
+            rank: The new rank value
+        """
+        self._rank = rank
