@@ -1,6 +1,6 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2020 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
@@ -40,7 +40,7 @@ class ExportProvider:
         Raises:
             Exception: If no appropriate strategy could be found
         """
-        strategy = config.INSTANCE.export_strategy
+        strategy = config.configuration.export_strategy
         if strategy in cls._strategies:
             exp = cls._strategies.get(strategy)
             assert exp, "Export strategy cannot be defined as None"
