@@ -1,15 +1,16 @@
 #  This file is part of Pynguin.
 #
-#  SPDX-FileCopyrightText: 2019–2020 Pynguin Contributors
+#  SPDX-FileCopyrightText: 2019–2021 Pynguin Contributors
 #
 #  SPDX-License-Identifier: LGPL-3.0-or-later
 #
 """Provides capabilities to track branch distances."""
 import dataclasses
 import logging
+import threading
 from math import inf
 from types import CodeType
-from typing import Any, Callable, Dict, Optional, Tuple
+from typing import Any, Callable, Dict, Optional, Set, Tuple
 
 from bytecode import Compare
 from jellyfish import levenshtein_distance
@@ -59,6 +60,11 @@ class KnownData:
         default_factory=dict
     )
 
+    # Stores which of the existing code objects do not contain a branch, i.e.,
+    # they do not contain a predicate. Every code object is initially seen as
+    # branch-less until a predicate is registered for it.
+    branch_less_code_objects: Set[int] = dataclasses.field(default_factory=set)
+
     # Maps all known ids of predicates to meta information
     existing_predicates: Dict[int, PredicateMetaData] = dataclasses.field(
         default_factory=dict
@@ -124,6 +130,22 @@ class ExecutionTracer:
         self._import_trace = ExecutionTrace()
         self._init_trace()
         self._enabled = True
+        self._current_thread_ident: Optional[int] = None
+
+    @property
+    def current_thread_ident(self) -> Optional[int]:
+        """Get the current thread ident."""
+        return self._current_thread_ident
+
+    @current_thread_ident.setter
+    def current_thread_ident(self, current: int) -> None:
+        """Set the current thread ident. Tracing calls from any other thread
+        are ignored.
+
+        Args:
+            current: the current thread
+        """
+        self._current_thread_ident = current
 
     def get_known_data(self) -> KnownData:
         """Provide known data.
@@ -201,6 +223,7 @@ class ExecutionTracer:
         """
         code_object_id = len(self._known_data.existing_code_objects)
         self._known_data.existing_code_objects[code_object_id] = meta
+        self._known_data.branch_less_code_objects.add(code_object_id)
         return code_object_id
 
     def executed_code_object(self, code_object_id: int) -> None:
@@ -212,6 +235,9 @@ class ExecutionTracer:
         Args:
             code_object_id: the code object id to mark
         """
+        if threading.currentThread().ident != self._current_thread_ident:
+            return
+
         assert (
             code_object_id in self._known_data.existing_code_objects
         ), "Cannot trace unknown code object"
@@ -229,6 +255,7 @@ class ExecutionTracer:
         """
         predicate_id = len(self._known_data.existing_predicates)
         self._known_data.existing_predicates[predicate_id] = meta
+        self._known_data.branch_less_code_objects.discard(meta.code_object_id)
         return predicate_id
 
     def executed_compare_predicate(
@@ -242,6 +269,9 @@ class ExecutionTracer:
             predicate: the predicate
             cmp_op: the compare operation
         """
+        if threading.currentThread().ident != self._current_thread_ident:
+            return
+
         if self._is_disabled():
             return
 
@@ -265,6 +295,9 @@ class ExecutionTracer:
             value: the value
             predicate: the predicate
         """
+        if threading.currentThread().ident != self._current_thread_ident:
+            return
+
         if self._is_disabled():
             return
 
